<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace TcbPay\Payment\Model\Payment;

class Simple extends \Magento\Payment\Model\Method\Cc
{
    const METHOD_CODE      = 'tcbpay_simple';
    const API_LIVE_URL     = 'https://gateway2.tcbpay.com/mobile_plugin/sale.php';
    const API_TEST_URL     = 'https://gateway2-stg.tcbpay.com/mobile_plugin/sale.php';

    /**
     * @var string
     */
    protected $_code  = self::METHOD_CODE;

    /**
     * Payment Method feature
     *
     * @var bool
     */
    protected $_isGateway                   = true;

    /**
     * Payment Method feature
     *
     * @var bool
     */
    protected $_canCapture                  = true;

    /**
     * Payment Method feature
     *
     * @var bool
     */
    protected $_canCapturePartial           = true;

    /**
     * Payment Method feature
     *
     * @var bool
     */
    protected $_canRefund                   = false;

    /**
     * Payment Method min order total
     *
     * @var int
     */
    protected $_minOrderTotal = 0;

    /**
     * Payment Method suported currencies
     *
     * @var array
     */
    protected $_supportedCurrencyCodes = array('USD','GBP','EUR');

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    protected $_curl;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param Logger $logger
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,        
        array $data = array()
    ) {
        $this->_curl = $curl;

        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate,
            $resource,
            $resourceCollection,
            $data
        );

        $this->_minOrderTotal = $this->getConfigData('min_order_total');
    }

    /**
     * Check method for processing with base currency
     *
     * @param string $currencyCode
     * @return bool
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function canUseForCurrency($currencyCode)
    {
       // if (!in_array($currencyCode, $this->_supportedCurrencyCodes)) {
          //  return false;
        //}
        return true;
    }

    /**
     * Capture payment abstract method
     *
     * @param \Magento\Framework\DataObject|InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @api
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        if ($amount <= 0) {
            throw new \Magento\Framework\Exception\LocalizedException(__('Invalid amount for authorization.'));
        }
       // $order = $payment->getOrder();
       // $billing = $order->getBillingAddress();

        try {
            $response = $this->_postRequest($payment, $amount);
            $resp = $response['resp'];

            if ($response['success']) {
                if (empty($resp->transaction_id)) {
                    throw new \Magento\Framework\Exception\LocalizedException(__('TCBPay\'s transaction id is empty.'));
                }
                $payment->setTransactionId($resp->transaction_id);
                //$payment->setParentTransactionId($resp->transaction_id);
                $payment->setIsTransactionClosed(1);
            } else {
                throw new \Magento\Framework\Exception\LocalizedException(__($resp->description));
            }
        } catch (\Exception $e) {
            throw new \Magento\Framework\Validator\Exception(__('Error: '.$e->getMessage()));
        }

        return $this;
    }

    /**
     * Refund specified amount for payment
     *
     * @param \Magento\Framework\DataObject|InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @api
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        if (!$this->canRefund()) {
            throw new \Magento\Framework\Exception\LocalizedException(__('The refund action is not available.'));
        }

        return $this;
    }

    /**
     * Prepare request to tcbpay gateway
     *
     * @param \Magento\Framework\DataObject|InfoInterface $payment
     * @param float $amount
     * @return array
     */
    protected function _postRequest(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        //$this->setStore($order->getStoreId());

        $temp = $payment->getCcExpYear();
        $CcExpYear = str_split($temp, 2);
        $params = array(
            "username"              => $this->getConfigData('username'),
            "api_key"               => $this->getConfigData('password'),
            "amount"                => $amount,
            "creditcard_number"     => $payment->getCcNumber(),
            "cvv"                   => ( $this->hasVerification() ? $payment->getCcCid() : ''),
            "expire_month"          => str_pad($payment->getCcExpMonth(),2,0,STR_PAD_LEFT),
            "expire_year"           => $CcExpYear[1],
            "first_name"            => $billing->getFirstname(),
            "last_name"             => $billing->getLastname(),
            "address"               => $billing->getStreetLine(1),
            "city"                  => $billing->getCity(),
            "state"                 => $billing->getRegion(),
            "zip_code"              => $billing->getPostcode(),
            "country_code"          => $billing->getCountryId(),
            "phone_number"          => $billing->getTelephone(),
            "email"                 => $order->getCustomerEmail(),
            "currency"              => $order->getOrderCurrencyCode(),
            "memo"                  => '',
			"client_ip"				=> $_SERVER['REMOTE_ADDR'],
            'source'                => 'API_MAGENTO'
        );

        $gmtime = strtotime(gmdate('Y-m-d H:i:s'));

        $headers = [
            'hash_timestamp: '. $gmtime,
            'hash: '. md5($params['username']."|".$params['api_key']."|".$this->getConfigData('secretkey')."|".$gmtime)
        ];

        $url = self::API_LIVE_URL;
        if ($this->getConfigData('testmode')) {
            $url = self::API_TEST_URL;
        }

        $post_query =  http_build_query($params);

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, true); 
        curl_setopt($ch,CURLOPT_POSTFIELDS, $post_query);
        $response = curl_exec($ch);

        curl_close ($ch);

        $result = ['success' => false, 'resp' => ''];
        $respObj = json_decode($response);

        if (isset($respObj) && !empty($respObj) && isset($respObj->response)) {
            $resp = $respObj->response;
            $result['resp'] = $resp;
            if($resp->code == '00') {
                $result['success'] = true;
            }
        }

        return $result;
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null) {
        if (!$this->getConfigData('username') || !$this->getConfigData('password')) {
            return false;
        }

        if ($quote && $quote->getBaseGrandTotal() < $this->_minOrderTotal) {
            return false;
        }

        return parent::isAvailable($quote);
    }
}